@@ -549,6 +549,8 @@ class S3FileSystem(AsyncFileSystem):
         self, path, refresh=False, max_items=None, delimiter="/", prefix=""
     ):
         bucket, key, _ = self.split_path(path)
+        if not prefix:
+            prefix = ""
         if key:
             prefix = key.lstrip("/") + "/" + prefix
         if path not in self.dircache or refresh or not delimiter:
@@ -898,13 +900,16 @@ class S3FileSystem(AsyncFileSystem):
 
     async def _put_file(self, lpath, rpath, chunksize=50 * 2 ** 20, **kwargs):
         bucket, key, _ = self.split_path(rpath)
-        if os.path.isdir(lpath) and key:
-            # don't make remote "directory"
-            return
+        if os.path.isdir(lpath):
+            if key:
+                # don't make remote "directory"
+                return
+            else:
+                await self._mkdir(lpath)
         size = os.path.getsize(lpath)
         with open(lpath, "rb") as f0:
             if size < min(5 * 2 ** 30, 2 * chunksize):
-                return await self._call_s3(
+                await self._call_s3(
                     "put_object", Bucket=bucket, Key=key, Body=f0, **kwargs
                 )
             else:
@@ -939,7 +944,9 @@ class S3FileSystem(AsyncFileSystem):
                     UploadId=mpu["UploadId"],
                     MultipartUpload={"Parts": parts},
                 )
-        self.invalidate_cache(rpath)
+        while rpath:
+            self.invalidate_cache(rpath)
+            rpath = self._parent(rpath)
 
     async def _get_file(self, rpath, lpath, version_id=None):
         bucket, key, vers = self.split_path(rpath)
@@ -1630,6 +1637,14 @@ class S3FileSystem(AsyncFileSystem):
             self.invalidate_cache(self._parent(path))
         await self._call_s3("delete_objects", kwargs, Bucket=bucket, Delete=delete_keys)
 
+    async def _rm_file(self, path, **kwargs):
+        bucket, key, _ = self.split_path(path)
+
+        try:
+            await self._call_s3("delete_object", Bucket=bucket, Key=key)
+        except ClientError as e:
+            raise translate_boto_error(e)
+
     async def _rm(self, path, recursive=False, **kwargs):
         if recursive and isinstance(path, str):
             bucket, key, _ = self.split_path(path)
