@@ -2140,6 +2140,9 @@ def test_find_with_prefix(s3):
     assert test_1s == [test_bucket_name + "/prefixes/test_1"] + [
         test_bucket_name + f"/prefixes/test_{cursor}" for cursor in range(10, 20)
     ]
+    assert s3.find(test_bucket_name + "/prefixes/") == s3.find(
+        test_bucket_name + "/prefixes/", prefix=None
+    )
 
 
 def test_list_after_find(s3):
@@ -2148,3 +2151,22 @@ def test_list_after_find(s3):
     s3.find("s3://test/2014-01-01.csv")
     after = s3.ls("s3://test")
     assert before == after
+
+
+def test_upload_recursive_to_bucket(s3, tmpdir):
+    # GH#491
+    folders = [os.path.join(tmpdir, d) for d in ["outer", "outer/inner"]]
+    files = [os.path.join(tmpdir, f) for f in ["outer/afile", "outer/inner/bfile"]]
+    for d in folders:
+        os.mkdir(d)
+    for f in files:
+        open(f, "w").write("hello")
+    s3.put(folders[0], "newbucket", recursive=True)
+
+
+def test_rm_file(s3):
+    target = test_bucket_name + "/to_be_removed/file"
+    s3.touch(target)
+    s3.rm_file(target)
+    assert not s3.exists(target)
+    assert not s3.exists(test_bucket_name + "/to_be_removed")
